@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011-2014 gnome-pomodoro contributors
+ * Copyright (c) 2011-2017 gnome-pomodoro contributors
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@ const Signals = imports.signals;
 const Atk = imports.gi.Atk;
 const Clutter = imports.gi.Clutter;
 const GLib = imports.gi.GLib;
+const GObject = imports.gi.GObject;
 const Meta = imports.gi.Meta;
 const St = imports.gi.St;
 const Pango = imports.gi.Pango;
@@ -35,6 +36,8 @@ const Lightbox = imports.ui.lightbox;
 const Main = imports.ui.main;
 const Tweener = imports.ui.tweener;
 
+const Params = imports.misc.params;
+
 const Extension = imports.misc.extensionUtils.getCurrentExtension();
 const Config = Extension.imports.config;
 const Timer = Extension.imports.timer;
@@ -60,14 +63,37 @@ const IDLE_TIME_TO_OPEN = 60000;
 const IDLE_TIME_TO_CLOSE = 600;
 const MIN_DISPLAY_TIME = 500;
 
-const FADE_IN_TIME = 180;
-const FADE_IN_OPACITY = 0.55;
+const FADE_IN_TIME = 200;
+const FADE_IN_OPACITY = 0.45;
 
-const FADE_OUT_TIME = 180;
+const FADE_OUT_TIME = 200;
 
 const OPEN_WHEN_IDLE_MIN_REMAINING_TIME = 3.0;
 
-const State = {
+// A single pass blur approximation, with hacky brightness param
+const BLUR_FRAGMENT_SHADER = '\
+uniform sampler2D tex; \
+uniform float x_step, y_step, factor, brightness; \
+\
+const float[7] weights = float[7](0.1452444, 0.1314128, 0.1115940, 0.0831466, 0.0543562,  0.0311780,  0.0156902); \
+const float[7] offsets = float[7](0.0000000, 2.4876390, 4.4711854, 6.4547903, 8.4384988, 10.4223371, 12.4063545); \
+\
+void main () { \
+  vec2 uv = vec2(cogl_tex_coord_in[0].st); \
+  vec2 pixel_size = vec2(x_step * factor, y_step * factor); \
+  vec4 color = vec4(0.0); \
+  color += weights[0] * texture2D(tex, uv); \
+  for (int i=1; i < 7; i++) { \
+    color += \
+      weights[i] * texture2D(tex, uv - pixel_size * offsets[i]) + \
+      weights[i] * texture2D(tex, uv + pixel_size * offsets[i]); \
+  } \
+  color.rgb *= 1.0 - (1.0 - sqrt(brightness)) * factor; \
+  cogl_color_out = color; \
+}';
+
+
+var State = {
     OPENED: 0,
     CLOSED: 1,
     OPENING: 2,
@@ -75,129 +101,163 @@ const State = {
 };
 
 
-const MessagesIndicator = new Lang.Class({
-    Name: 'PomodoroMessagesIndicator',
+var BlurEffect = new Lang.Class({
+    Name: 'PmodoroBlurEffect',
+    Extends: Clutter.ShaderEffect,
 
-    _init: function() {
-        this._count = 0;
-        this._sources = [];
+    _init(params) {
+        params = Params.parse(params, { orientation: Clutter.Orientation.HORIZONTAL,
+                                        brightness: 1.0,
+                                        factor: 1.0 });
 
-        this._container = new St.BoxLayout({ style_class: 'messages-indicator-contents',
-                                             x_expand: true,
-                                             y_expand: true,
-                                             x_align: Clutter.ActorAlign.CENTER });
+        this.parent({ shader_type: Clutter.ShaderType.FRAGMENT_SHADER });
 
-        this._icon = new St.Icon({ icon_name: 'user-idle-symbolic',
-                                   icon_size: 16 });
-        this._container.add_actor(this._icon);
+        this.orientation = params.orientation;
+        this.brightness = params.brightness;
+        this.factor = params.factor;
 
-        this._label = new St.Label();
-        this._container.add_actor(this._label);
-
-        let layout = new Clutter.BinLayout();
-        this.actor = new St.Widget({ layout_manager: layout,
-                                     style_class: 'messages-indicator',
-                                     y_expand: true,
-                                     y_align: Clutter.ActorAlign.END,
-                                     visible: false });
-        this.actor.add_actor(this._container);
-        this.actor.connect('destroy', Lang.bind(this, this._onActorDestroy));
-
-        this._onSourceAddedId = Main.messageTray.connect('source-added', Lang.bind(this, this._onSourceAdded));
-        this._onSourceRemovedId = Main.messageTray.connect('source-removed', Lang.bind(this, this._onSourceRemoved));
-
-        let sources = Main.messageTray.getSources();
-        sources.forEach(Lang.bind(this, function(source) { this._onSourceAdded(null, source); }));
-
-        this._overviewShowingId = Main.overview.connect('showing', Lang.bind(this, this._updateVisibility));
+        this.set_shader_source(BLUR_FRAGMENT_SHADER);
     },
 
-    _onSourceAdded: function(tray, source) {
-        if (source.trayIcon) {
-            return;
-        }
-
-        let sourceInfo = {
-            source: source,
-            countUpdatedId: source.connect('count-updated', Lang.bind(this, this._updateCount))
-        };
-        this._sources.push(sourceInfo);
-        this._updateCount();
+    get brightness() {
+        return this._brightness;
     },
 
-    _onSourceRemoved: function(tray, source) {
-        let index, sourceInfo;
+    set brightness(value) {
+        this.set_uniform_value('brightness', GObject.Float(value));
+        this._brightness = value;
+    },
 
-        for (index=0; index < this._sources.length; index++) {
-            sourceInfo = this._sources[index];
-            if (sourceInfo.source === source) {
-                sourceInfo.source.disconnect(sourceInfo.countUpdatedId);
-                this._sources.splice(index, 1);
-                this._updateCount();
-                break;
+    get factor() {
+        return this._factor;
+    },
+
+    set factor(value) {
+        this.set_uniform_value('factor', GObject.Float(value));
+        this._factor = value;
+    },
+
+    vfunc_pre_paint() {
+        let res = this.parent();
+        let [success, width, height] = this.get_target_size();
+
+        if (success) {
+            // Actually we don't blur horizontally / vertically, but diagonally
+            if (this.orientation != Clutter.Orientation.HORIZONTAL) {
+                this.set_uniform_value('x_step', GObject.Float(1.0 / width));
+                this.set_uniform_value('y_step', GObject.Float(1.0 / height));
+            }
+            else {
+                this.set_uniform_value('x_step', - GObject.Float(1.0 / width));
+                this.set_uniform_value('y_step', GObject.Float(1.0 / height));
             }
         }
-    },
-
-    _updateCount: function() {
-        let count = 0;
-        let hasChats = false;
-        this._sources.forEach(Lang.bind(this,
-            function(sourceInfo) {
-                count += sourceInfo.source.unseenCount || sourceInfo.source.indicatorCount;
-                hasChats |= sourceInfo.source.isChat;
-            }));
-
-        this._count = count;
-
-        if (this._label.clutter_text) {
-            this._label.clutter_text.set_text(ngettext("%d new message",
-                                                       "%d new messages",
-                                                       count).format(count));
+        else {
+            this.set_uniform_value('x_step', GObject.Float(0.0));
+            this.set_uniform_value('y_step', GObject.Float(0.0));
         }
 
-        this._icon.visible = hasChats;
-        this._updateVisibility();
-    },
+        return res;
+    }
+});
+
+
+var BlurredLightbox = new Lang.Class({
+    Name: 'PomodoroBlurredLightbox',
+    Extends: Lightbox.Lightbox,
+
+    _init(container, params) {
+        params.radialEffect = false;
 
-    _updateVisibility: function() {
-        let visible = (this._count > 0);
+        this.parent(container, params);
 
-        this.actor.visible = visible;
-    },
+        if (Clutter.feature_available(Clutter.FeatureFlags.SHADERS_GLSL)) {
+            // TODO: Try consolidate these effects into one
+            this._blurXEffect = new BlurEffect({ orientation: Clutter.Orientation.HORIZONTAL,
+                                                 brightness: this._fadeFactor,
+                                                 factor: 0.0 });
+            this._blurYEffect = new BlurEffect({ orientation: Clutter.Orientation.VERTICAL,
+                                                 brightness: this._fadeFactor,
+                                                 factor: 0.0 });
 
-    _disconnectSignals: function() {
-        let sourceInfo;
+            // Clone the group that contains all of UI on the screen.  This is the
+            // chrome, the windows, etc.
+            let uiGroupClone = new Clutter.Clone({ source: Main.layoutManager.uiGroup,
+                                                   clip_to_allocation: true });
+            uiGroupClone.add_effect(this._blurXEffect);
+            uiGroupClone.add_effect(this._blurYEffect);
 
-        if (this._onSourceAddedId) {
-            Main.messageTray.disconnect(this._onSourceAddedId);
-            this._onSourceAddedId = 0;
+            this.actor.add_actor(uiGroupClone);
+            this.actor.opacity = 255;
         }
-        if (this._onSourceRemovedId) {
-            Main.messageTray.disconnect(this._onSourceRemovedId);
-            this._onSourceRemovedId = 0;
-        }
+
+        this.actor.add_style_class_name('extension-pomodoro-lightbox');
+    },
+
+    show(fadeInTime) {
+        fadeInTime = fadeInTime || 0;
 
-        if (this._overviewShowingId) {
-            Main.overview.disconnect(this._overviewShowingId);
-            this._overviewShowingId = 0;
+        if (this._blurXEffect) {
+            Tweener.removeTweens(this._blurXEffect);
+            Tweener.addTween(this._blurXEffect,
+                             { factor: 1.0,
+                               time: fadeInTime,
+                               transition: 'easeOutQuad',
+                               onUpdate: () => {
+                                   this._blurYEffect.factor = this._blurXEffect.factor;
+                               },
+                               onComplete: () => {
+                                   this.shown = true;
+                                   this.emit('shown');
+                               }
+                             });
+        } else {
+            Tweener.removeTweens(this.actor);
+            Tweener.addTween(this.actor,
+                             { opacity: 255 * this._fadeFactor,
+                               time: fadeInTime,
+                               transition: 'easeOutQuad',
+                               onComplete: () => {
+                                   this.shown = true;
+                                   this.emit('shown');
+                               }
+                             });
         }
 
-        while (this._sources.length > 0) {
-            sourceInfo = this._sources.pop();
-            sourceInfo.source.disconnect(sourceInfo.countUpdatedId);
-        }
+        this.actor.show();
     },
 
-    _onActorDestroy: function() {
-        if (this._destroyed)
-            return;
-        this._destroyed = true;
-        this._disconnectSignals();
-        this.emit('destroy');
+    hide(fadeOutTime) {
+        fadeOutTime = fadeOutTime || 0;
+
+        this.shown = false;
+
+        if (this._blurXEffect) {
+            Tweener.removeTweens(this._blurXEffect);
+            Tweener.addTween(this._blurXEffect,
+                             { factor: 0.0,
+                               time: fadeOutTime,
+                               transition: 'easeOutQuad',
+                               onUpdate: () => {
+                                   this._blurYEffect.factor = this._blurXEffect.factor;
+                               },
+                               onComplete: () => {
+                                   this.actor.hide();
+                               }
+                             });
+        } else {
+            Tweener.removeTweens(this.actor);
+            Tweener.addTween(this.actor,
+                             { opacity: 0,
+                               time: fadeOutTime,
+                               transition: 'easeOutQuad',
+                               onComplete: () => {
+                                   this.actor.hide();
+                               }
+                             });
+        }
     }
 });
-Signals.addSignalMethods(MessagesIndicator.prototype);
 
 
 /**
@@ -205,10 +265,10 @@ Signals.addSignalMethods(MessagesIndicator.prototype);
  * class to have more event signals, different fade in/out times, and different
  * event blocking behavior.
  */
-const ModalDialog = new Lang.Class({
+var ModalDialog = new Lang.Class({
     Name: 'PomodoroModalDialog',
 
-    _init: function() {
+    _init() {
         this.state = State.CLOSED;
 
         this._idleMonitor          = Meta.IdleMonitor.get_core();
@@ -218,52 +278,37 @@ const ModalDialog = new Lang.Class({
         this._pushModalTries       = 0;
 
         this._monitorConstraint = new Layout.MonitorConstraint();
-        this._stageConstraint   = new Clutter.BindConstraint({
+        this._stageConstraint = new Clutter.BindConstraint({
                                        source: global.stage,
                                        coordinate: Clutter.BindCoordinate.ALL });
 
-        this._layout = new St.BoxLayout({ vertical: true });
-
-        /* Modal dialogs are fixed width and grow vertically; set the request
-         * mode accordingly so wrapped labels are handled correctly during
-         * size requests.
-         */
-        this._layout.request_mode = Clutter.RequestMode.HEIGHT_FOR_WIDTH;
-
-        this._backgroundStack = new St.Widget({ layout_manager: new Clutter.BinLayout() });
-        this._backgroundStack.add_actor(this._layout);
-
-
-        let backgroundBin = new St.Bin({ child: this._backgroundStack,
-                                         x_fill: true,
-                                         y_fill: true });
-        backgroundBin.add_constraint(this._monitorConstraint);
-
-        this.actor = new St.Widget({ accessible_role: Atk.Role.DIALOG,
+        this.actor = new St.Widget({ style_class: 'extension-pomodoro-dialog',
+                                     accessible_role: Atk.Role.DIALOG,
+                                     layout_manager: new Clutter.BinLayout(),
                                      visible: false,
-                                     opacity: 0.0,
-                                     x: 0,
-                                     y: 0 });
+                                     opacity: 0 });
         this.actor._delegate = this;
         this.actor.add_constraint(this._stageConstraint);
-        this.actor.add_style_class_name('extension-pomodoro-dialog');
-        this.actor.add_actor(backgroundBin);
-        this.actor.connect('destroy', Lang.bind(this, this._onActorDestroy));
+        this.actor.connect('destroy', this._onActorDestroy.bind(this));
 
-        // TODO: Check it it's ok with multi-monitor setup
-        global.stage.add_actor(this.actor);
-        // Main.layoutManager.modalDialogGroup.add_actor(this.actor);
+        // Modal dialogs are fixed width and grow vertically; set the request
+        // mode accordingly so wrapped labels are handled correctly during
+        // size requests.
+        this._layout = new St.Widget({ layout_manager: new Clutter.BinLayout() });
+        this._layout.add_constraint(this._monitorConstraint);
 
-        this._lightbox = new Lightbox.Lightbox(this.actor,
-                                               { fadeFactor: FADE_IN_OPACITY,
-                                                 inhibitEvents: false });
-        this._lightbox.highlight(backgroundBin);
-        this._lightbox.actor.add_style_class_name('extension-pomodoro-lightbox');
-        this._lightbox.show();
+        this.actor.add_actor(this._layout);
+
+        this._lightbox = new BlurredLightbox(this.actor,
+                                             { fadeFactor: FADE_IN_OPACITY,
+                                               inhibitEvents: false });
+        this._lightbox.highlight(this._layout);
+        this._lightbox.actor.show();
 
         this._grabHelper = new GrabHelper.GrabHelper(this.actor);
         this._grabHelper.addActor(this._lightbox.actor);
 
+        global.stage.add_actor(this.actor);
         global.focus_manager.add_group(this.actor);
     },
 
@@ -271,7 +316,32 @@ const ModalDialog = new Lang.Class({
         return this.state == State.OPENED || this.state == State.OPENING;
     },
 
-    open: function(animate) {
+    _addMessageTray() {
+        let messageTray = Main.messageTray;
+
+        messageTray.actor.ref();
+
+        Main.layoutManager.removeChrome(messageTray.actor);
+
+        global.stage.add_child(messageTray.actor);
+
+        messageTray.actor.unref();
+        messageTray.bannerBlocked = false;
+    },
+
+    _removeMessageTray() {
+        let messageTray = Main.messageTray;
+
+        messageTray.actor.ref();
+
+        global.stage.remove_child(messageTray.actor);
+
+        Main.layoutManager.addChrome(messageTray.actor, { affectsInputRegion: false });
+
+        messageTray.actor.unref();
+    },
+
+    open(animate) {
         if (this.state == State.OPENED || this.state == State.OPENING) {
             return;
         }
@@ -281,42 +351,47 @@ const ModalDialog = new Lang.Class({
         if (this._pushModalDelaySource == 0) {
             this._pushModalDelaySource = Mainloop.timeout_add(
                         Math.max(MIN_DISPLAY_TIME - IDLE_TIME_TO_PUSH_MODAL, 0),
-                        Lang.bind(this, this._onPushModalDelayTimeout));
+                        this._onPushModalDelayTimeout.bind(this));
         }
 
-        this._monitorConstraint.index = global.screen.get_current_monitor();
+        // fallback to global.screen.get_current_monitor() for mutter < 3.29
+        this._monitorConstraint.index = typeof(global.display) === 'object' && typeof(global.display.get_current_monitor) !== 'undefined'
+            ? global.display.get_current_monitor() : global.screen.get_current_monitor();
 
         this.actor.raise_top();
         this.actor.show();
 
         Tweener.removeTweens(this.actor);
 
+        this._addMessageTray();
+
         if (animate) {
+            this._lightbox.show(FADE_IN_TIME / 1000);
             Tweener.addTween(this.actor,
                              { opacity: 255,
                                time: FADE_IN_TIME / 1000,
                                transition: 'easeOutQuad',
-                               onComplete: Lang.bind(this,
-                                    function() {
-                                        if (this.state == State.OPENING) {
-                                            this.state = State.OPENED;
-                                            this.emit('opened');
-                                        }
-                                    })
+                               onComplete: () => {
+                                   if (this.state == State.OPENING) {
+                                       this.state = State.OPENED;
+                                       this.emit('opened');
+                                   }
+                               }
                              });
             this.emit('opening');
         }
         else {
-            this.state = State.OPENED;
-
+            this._lightbox.show();
             this.actor.opacity = 255;
 
+            this.state = State.OPENED;
+
             this.emit('opening');
             this.emit('opened');
         }
     },
 
-    close: function(animate) {
+    close(animate) {
         this._cancelOpenWhenIdle();
 
         if (this.state == State.CLOSED || this.state == State.CLOSING) {
@@ -329,53 +404,58 @@ const ModalDialog = new Lang.Class({
         Tweener.removeTweens(this.actor);
 
         if (animate) {
+            this._lightbox.hide(FADE_OUT_TIME / 1000);
             Tweener.addTween(this.actor,
                              { opacity: 0,
                                time: FADE_OUT_TIME / 1000,
                                transition: 'easeOutQuad',
-                               onComplete: Lang.bind(this,
-                                   function() {
-                                        if (this.state == State.CLOSING) {
-                                            this.state = State.CLOSED;
-                                            this.actor.hide();
-                                            this.emit('closed');
-                                        }
-                                   })
+                               onComplete: () => {
+                                   if (this.state == State.CLOSING) {
+                                       this.state = State.CLOSED;
+                                       this.actor.hide();
+
+                                       this._removeMessageTray();
+
+                                       this.emit('closed');
+                                   }
+                               }
                              });
             this.emit('closing');
         }
         else {
-            this.state = State.CLOSED;
-
             this.actor.opacity = 0;
             this.actor.hide();
+            this._lightbox.hide();
+
+            this.state = State.CLOSED;
+
+            this._removeMessageTray();
 
             this.emit('closing');
             this.emit('closed');
         }
     },
 
-    _onPushModalDelayTimeout: function() {
+    _onPushModalDelayTimeout() {
         /* Don't become modal and block events just yet,
          * wait until user becomes idle.
          */
         if (this._pushModalWatchId == 0) {
-            this._pushModalWatchId = this._idleMonitor.add_idle_watch(IDLE_TIME_TO_PUSH_MODAL, Lang.bind(this,
-                function(monitor) {
+            this._pushModalWatchId = this._idleMonitor.add_idle_watch(IDLE_TIME_TO_PUSH_MODAL,
+                (monitor) => {
                     if (this._pushModalWatchId) {
                         this._idleMonitor.remove_watch(this._pushModalWatchId);
                         this._pushModalWatchId = 0;
                     }
                     this.pushModal();
-                }
-            ));
+                });
         }
 
         this._pushModalDelaySource = 0;
         return GLib.SOURCE_REMOVE;
     },
 
-    _pushModal: function() {
+    _pushModal() {
         if (this.state == State.CLOSED || this.state == State.CLOSING) {
             return false;
         }
@@ -383,11 +463,11 @@ const ModalDialog = new Lang.Class({
         return this._grabHelper.grab({
             actor: this._lightbox.actor,
             focus: this._lightbox.actor,
-            onUngrab: Lang.bind(this, this._onUngrab)
+            onUngrab: this._onUngrab.bind(this)
         });
     },
 
-    _onPushModalTimeout: function() {
+    _onPushModalTimeout() {
         if (this.state == State.CLOSED || this.state == State.CLOSING) {
             this._pushModalSource = 0;
             return GLib.SOURCE_REMOVE;
@@ -409,7 +489,7 @@ const ModalDialog = new Lang.Class({
         return GLib.SOURCE_CONTINUE;
     },
 
-    pushModal: function() {
+    pushModal() {
         if (this.state == State.CLOSED || this.state == State.CLOSING) {
             return;
         }
@@ -421,8 +501,8 @@ const ModalDialog = new Lang.Class({
         // this._grabHelper.ignoreRelease();
 
         /* delay pushModal to ignore current events */
-        Mainloop.idle_add(Lang.bind(this,
-            function() {
+        Mainloop.idle_add(
+            () => {
                 this._pushModalTries = 1;
 
                 if (this._pushModal()) {
@@ -430,12 +510,11 @@ const ModalDialog = new Lang.Class({
                 }
                 else {
                     this._pushModalSource = Mainloop.timeout_add(Math.floor(1000 / PUSH_MODAL_RATE),
-                                                                 Lang.bind(this, this._onPushModalTimeout));
+                                                                 this._onPushModalTimeout.bind(this));
                 }
 
                 return GLib.SOURCE_REMOVE;
-            }
-        ));
+            });
     },
 
     /**
@@ -443,7 +522,7 @@ const ModalDialog = new Lang.Class({
      * dialog insensitive as well, so it needs to be followed shortly
      * by either a close() or a pushModal()
      */
-    popModal: function() {
+    popModal() {
         try {
             if (this._grabHelper.isActorGrabbed(this._lightbox.actor))
             {
@@ -459,7 +538,7 @@ const ModalDialog = new Lang.Class({
         this._disconnectSignals();
     },
 
-    _disconnectSignals: function() {
+    _disconnectSignals() {
         if (this._pushModalDelaySource) {
             Mainloop.source_remove(this._pushModalDelaySource);
             this._pushModalDelaySource = 0;
@@ -476,11 +555,11 @@ const ModalDialog = new Lang.Class({
         }
     },
 
-    _onUngrab: function() {
+    _onUngrab() {
         this.close(true);
     },
 
-    _onActorDestroy: function() {
+    _onActorDestroy() {
         if (this._destroyed)
             return;
         this._destroyed = true;
@@ -492,18 +571,18 @@ const ModalDialog = new Lang.Class({
         this.emit('destroy');
     },
 
-    destroy: function() {
+    destroy() {
         this.actor.destroy();
     }
 });
 Signals.addSignalMethods(ModalDialog.prototype);
 
 
-const PomodoroEndDialog = new Lang.Class({
+var PomodoroEndDialog = new Lang.Class({
     Name: 'PomodoroEndDialog',
     Extends: ModalDialog,
 
-    _init: function(timer) {
+    _init(timer) {
         this.parent();
 
         this.timer = timer;
@@ -532,32 +611,18 @@ const PomodoroEndDialog = new Lang.Class({
         box.add(this._descriptionLabel,
                 { y_fill: false,
                   y_align: St.Align.START });
-        this._layout.add(box,
-                         { expand: true,
-                           x_fill: false,
-                           y_fill: false,
-                           x_align: St.Align.MIDDLE,
-                           y_align: St.Align.MIDDLE });
+        this._layout.add_actor(box);
 
-        let messagesIndicator = new MessagesIndicator();
-        this._backgroundStack.add_actor(messagesIndicator.actor);
-//        this._layout.add(messagesIndicator.actor,
-//                         { expand: false,
-//                           x_fill: false,
-//                           y_fill: false,
-//                           x_align: St.Align.MIDDLE,
-//                           y_align: St.Align.END });
+        this._actorMappedId = this.actor.connect('notify::mapped', this._onActorMappedChanged.bind(this));
 
-        this._actorMappedId = this.actor.connect('notify::mapped', Lang.bind(this, this._onActorMappedChanged));
-
-        this.connect('closing', Lang.bind(this, this._onClosing));
-        this.connect('destroy', Lang.bind(this, this._onDestroy));
+        this.connect('closing', this._onClosing.bind(this));
+        this.connect('destroy', this._onDestroy.bind(this));
     },
 
-    _onActorMappedChanged: function(actor) {
+    _onActorMappedChanged(actor) {
         if (actor.mapped) {
             if (!this._timerUpdateId) {
-                this._timerUpdateId = this.timer.connect('update', Lang.bind(this, this._onTimerUpdate));
+                this._timerUpdateId = this.timer.connect('update', this._onTimerUpdate.bind(this));
                 this._onTimerUpdate();
             }
         }
@@ -569,7 +634,7 @@ const PomodoroEndDialog = new Lang.Class({
         }
     },
 
-    _onTimerUpdate: function() {
+    _onTimerUpdate() {
         if (this.timer.isBreak()) {
             let remaining = Math.max(this.timer.getRemaining(), 0.0);
             let minutes   = Math.floor(remaining / 60);
@@ -582,7 +647,7 @@ const PomodoroEndDialog = new Lang.Class({
         }
     },
 
-    _onClosing: function() {
+    _onClosing() {
         this._cancelCloseWhenActive();
         this._cancelOpenWhenIdle();
 
@@ -602,7 +667,7 @@ const PomodoroEndDialog = new Lang.Class({
         }
     },
 
-    _onDestroy: function() {
+    _onDestroy() {
         this._onClosing();
 
         if (this._actorMappedId) {
@@ -611,7 +676,7 @@ const PomodoroEndDialog = new Lang.Class({
         }
     },
 
-    _onEvent: function(actor, event) {
+    _onEvent(actor, event) {
         let x, y, dx, dy, distance;
 
         if (!event.get_device ()) {
@@ -670,18 +735,18 @@ const PomodoroEndDialog = new Lang.Class({
     /**
      * Open the dialog and setup closing when user becomes active.
      */
-    open: function(animate) {
+    open(animate) {
         this.parent(animate);
 
         /* Wait until user has a chance of seeing the dialog */
         if (this._closeWhenActiveDelaySource == 0) {
-            this._closeWhenActiveDelaySource = Mainloop.timeout_add(MIN_DISPLAY_TIME, Lang.bind(this,
-                function() {
+            this._closeWhenActiveDelaySource = Mainloop.timeout_add(MIN_DISPLAY_TIME,
+                () => {
                     if (this._idleMonitor.get_idletime() < IDLE_TIME_TO_CLOSE) {
                         /* Wait until user becomes slightly idle */
                         this._closeWhenActiveIdleWatchId =
                                 this._idleMonitor.add_idle_watch(IDLE_TIME_TO_CLOSE,
-                                                                 Lang.bind(this, this.closeWhenActive));
+                                                                 this.closeWhenActive.bind(this));
                     }
                     else {
                         this.closeWhenActive();
@@ -689,25 +754,25 @@ const PomodoroEndDialog = new Lang.Class({
 
                     this._closeWhenActiveDelaySource = 0;
                     return GLib.SOURCE_REMOVE;
-                }));
+                });
         }
     },
 
-    _cancelOpenWhenIdle: function() {
+    _cancelOpenWhenIdle() {
         if (this._openWhenIdleWatchId) {
             this._idleMonitor.remove_watch(this._openWhenIdleWatchId);
             this._openWhenIdleWatchId = 0;
         }
     },
 
-    openWhenIdle: function() {
+    openWhenIdle() {
         if (this.state == State.OPEN || this.state == State.OPENING) {
             return;
         }
 
         if (this._openWhenIdleWatchId == 0) {
-            this._openWhenIdleWatchId = this._idleMonitor.add_idle_watch(IDLE_TIME_TO_OPEN, Lang.bind(this,
-                function() {
+            this._openWhenIdleWatchId = this._idleMonitor.add_idle_watch(IDLE_TIME_TO_OPEN,
+                () => {
                     let info = Utils.getFocusedWindowInfo();
 
                     if (info.isPlayer && info.isFullscreen)
@@ -723,12 +788,11 @@ const PomodoroEndDialog = new Lang.Class({
                     }
 
                     this.open(true);
-                }
-            ));
+                });
         }
     },
 
-    _cancelCloseWhenActive: function() {
+    _cancelCloseWhenActive() {
         if (this._eventId) {
             this._lightbox.actor.disconnect(this._eventId);
             this._eventId = 0;
@@ -736,7 +800,7 @@ const PomodoroEndDialog = new Lang.Class({
     },
 
     // TODO: should be private
-    closeWhenActive: function() {
+    closeWhenActive() {
         if (this.state == State.CLOSED || this.state == State.CLOSING) {
             return;
         }
@@ -744,11 +808,11 @@ const PomodoroEndDialog = new Lang.Class({
         if (this._eventId == 0) {
             this._eventX = -1;
             this._eventY = -1;
-            this._eventId = this._lightbox.actor.connect('event', Lang.bind(this, this._onEvent));
+            this._eventId = this._lightbox.actor.connect('event', this._onEvent.bind(this));
         }
     },
 
-    setDescription: function(text) {
+    setDescription(text) {
         this.description = text;
 
         if (this._descriptionLabel.clutter_text) {
