@@ -1,7 +1,7 @@
 /*
  * A simple pomodoro timer for GNOME Shell
  *
- * Copyright (c) 2011-2014 gnome-pomodoro contributors
+ * Copyright (c) 2011-2017 gnome-pomodoro contributors
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,10 +28,12 @@ const Gio = imports.gi.Gio;
 const Main = imports.ui.main;
 const Meta = imports.gi.Meta;
 const Shell = imports.gi.Shell;
+const ExtensionUtils = imports.misc.extensionUtils;
 const ExtensionSystem = imports.ui.extensionSystem;
 
-const Extension = imports.misc.extensionUtils.getCurrentExtension();
+const Extension = ExtensionUtils.getCurrentExtension();
 const Config = Extension.imports.config;
+const DBus = Extension.imports.dbus;
 const Indicator = Extension.imports.indicator;
 const Notifications = Extension.imports.notifications;
 const Dialogs = Extension.imports.dialogs;
@@ -41,53 +43,58 @@ const Timer = Extension.imports.timer;
 const Utils = Extension.imports.utils;
 
 
-let extension = null;
+// notifications pop up before state changes
+const NOTIFICATIONS_TIME_OFFSET = 10.0;
 
 
-const ExtensionMode = {
+var ExtensionMode = {
     DEFAULT: 0,
     RESTRICTED: 1
 };
 
 
-const PomodoroExtension = new Lang.Class({
+var PomodoroExtension = new Lang.Class({
     Name: 'PomodoroExtension',
 
-    _init: function(mode) {
-        Extension.extension = this;
-
-        this.settings           = null;
-        this.pluginSettings     = null;
-        this.timer              = null;
-        this.indicator          = null;
-        this.notificationSource = null;
-        this.notification       = null;
-        this.dialog             = null;
-        this.reminder           = null;
-        this.presence           = null;
-        this.mode               = null;
-        this.keybinding         = false;
-        this._isPaused          = false;
-        this._timerState        = Timer.State.NULL;
+    _init(mode) {
+        this.settings            = null;
+        this.pluginSettings      = null;
+        this.timer               = null;
+        this.indicator           = null;
+        this.notificationSource  = null;
+        this.notification        = null;
+        this.dialog              = null;
+        this.presence            = null;
+        this.mode                = null;
+        this.service             = null;
+        this.keybinding          = false;
+        this._pendingMode        = false;
+        this._isPaused           = false;
+        this._timerState         = Timer.State.NULL;
+        this._timerStateDuration = 0.0;
 
         try {
             this.settings = Settings.getSettings('org.gnome.pomodoro.preferences');
-            this.settings.connect('changed::show-reminders',
-                                  Lang.bind(this, this._onSettingsChanged));
             this.settings.connect('changed::show-screen-notifications',
-                                  Lang.bind(this, this._onSettingsChanged));
+                                  this._onSettingsChanged.bind(this));
 
             this.pluginSettings = Settings.getSettings('org.gnome.pomodoro.plugins.gnome');
             this.pluginSettings.connect('changed::hide-system-notifications',
-                                        Lang.bind(this, this._onSettingsChanged));
+                                        this._onSettingsChanged.bind(this));
             this.pluginSettings.connect('changed::indicator-type',
-                                        Lang.bind(this, this._onSettingsChanged));
+                                        this._onSettingsChanged.bind(this));
 
             this.timer = new Timer.Timer();
-            this.timer.connect('service-disconnected', Lang.bind(this, this._onServiceDisconnected));
-            this.timer.connect('state-changed', Lang.bind(this, this._onTimerStateChanged));
-            this.timer.connect('paused', Lang.bind(this, this._onTimerPaused));
-            this.timer.connect('resumed', Lang.bind(this, this._onTimerResumed));
+            this.timer.connect('service-connected', this._onTimerServiceConnected.bind(this));
+            this.timer.connect('service-disconnected', this._onTimerServiceDisconnected.bind(this));
+            this.timer.connect('update', this._onTimerUpdate.bind(this));
+            this.timer.connect('state-changed', this._onTimerStateChanged.bind(this));
+            this.timer.connect('paused', this._onTimerPaused.bind(this));
+            this.timer.connect('resumed', this._onTimerResumed.bind(this));
+
+            this.service = new DBus.PomodoroExtension();
+            this.service.connect('name-acquired', this._onServiceNameAcquired.bind(this));
+            this.service.connect('name-lost', this._onServiceNameLost.bind(this));
 
             this.setMode(mode);
         }
@@ -100,24 +107,27 @@ const PomodoroExtension = new Lang.Class({
         return Shell.AppSystem.get_default().lookup_app('org.gnome.Pomodoro.desktop');
     },
 
-    setMode: function(mode) {
-        if (this.mode !== mode) {
+    setMode(mode) {
+        if (!this.service.initialized) {
             this.mode = mode;
+            this._isModePending = true;
+
+            return;  /* wait until service name is acquired */
+        }
+
+        if (this.mode != mode || this._isModePending) {
+            this.mode = mode;
+            this._isModePending = false;
 
             if (mode == ExtensionMode.RESTRICTED) {
                 this._disableIndicator();
-                this._disableScreenNotifications();
-                this._disableReminders();
+                this._disableScreenNotification();
             }
             else {
                 this._enableIndicator();
 
                 if (this.settings.get_boolean('show-screen-notifications')) {
-                    this._enableScreenNotifications();
-                }
-
-                if (this.settings.get_boolean('show-reminders')) {
-                    this._enableReminders();
+                    this._enableScreenNotification();
                 }
             }
 
@@ -129,33 +139,23 @@ const PomodoroExtension = new Lang.Class({
             }
 
             this._enableKeybinding();
-            this._updateNotifications();
+            this._updateNotification();
         }
     },
 
-    notifyIssue: function(message) {
+    notifyIssue(message) {
         let notification = new Notifications.IssueNotification(message);
         notification.show();
     },
 
-    _onSettingsChanged: function(settings, key) {
+    _onSettingsChanged(settings, key) {
         switch(key) {
             case 'show-screen-notifications':
                 if (settings.get_boolean(key) && this.mode != ExtensionMode.RESTRICTED) {
-                    this._enableScreenNotifications();
+                    this._enableScreenNotification();
                 }
                 else {
-                    this._disableScreenNotifications();
-                }
-
-                break;
-
-            case 'show-reminders':
-                if (settings.get_boolean(key) && this.mode != ExtensionMode.RESTRICTED) {
-                    this._enableReminders();
-                }
-                else {
-                    this._disableReminders();
+                    this._disableScreenNotification();
                 }
 
                 break;
@@ -179,115 +179,163 @@ const PomodoroExtension = new Lang.Class({
         }
     },
 
-    _onServiceDisconnected: function() {
-        Utils.disableExtension(Config.EXTENSION_UUID);
+    _onServiceNameAcquired() {
+        this.emit('service-name-acquired');
+
+        this.setMode(this.mode);
     },
 
-    _onTimerPaused: function() {
+    _onServiceNameLost() {
+        this.emit('service-name-lost');
+
+        Utils.logError('Lost service name "org.gnome.Pomodoro.Extension"');
+    },
+
+    _onTimerServiceConnected() {
+        this.service.run();
+    },
+
+    _onTimerServiceDisconnected() {
+        Utils.logWarning('Lost connection to "org.gnome.Pomodoro"');
+    },
+
+    _onTimerUpdate() {
+        let remaining = this.timer.getRemaining();
+
+        if (remaining <= NOTIFICATIONS_TIME_OFFSET && !this.notification) {
+            this._updateNotification();
+        }
+    },
+
+    _onTimerPaused() {
         this._update();
     },
 
-    _onTimerResumed: function() {
+    _onTimerResumed() {
         this._update();
     },
 
-    _onTimerStateChanged: function() {
+    _onTimerStateChanged() {
         this._update();
     },
 
-    _onKeybindingPressed: function() {
+    _onKeybindingPressed() {
         if (this.timer) {
             this.timer.toggle();
         }
     },
 
-    _onNotificationDestroy: function(notification) {
+    _onNotificationDestroy(notification) {
         if (this.notification === notification) {
             this.notification = null;
         }
     },
 
-    _notifyPomodoroStart: function() {
+    _notifyPomodoroStart() {
         if (this.notification &&
             this.notification instanceof Notifications.PomodoroStartNotification)
         {
-            /* do not renotify */
+            if (this.notification.resident || this.notification.acknowledged) {
+                this.notification.show();
+            }
         }
         else {
             this.notification = new Notifications.PomodoroStartNotification(this.timer);
-            this.notification.connect('activated', Lang.bind(this,
-                function(notification) {
-                    this.application.activate();
-                }));
-            this.notification.connect('destroy', Lang.bind(this, this._onNotificationDestroy));
+            this.notification.connect('activated',
+                (notification) => {
+                    if (this.timer.isBreak()) {
+                        this.timer.skip();
+                    }
+                    else {
+                        notification.destroy();
+                    }
+                });
+            this.notification.connect('destroy', this._onNotificationDestroy.bind(this));
             this.notification.show();
 
             this._destroyPreviousNotifications();
         }
     },
 
-    _notifyPomodoroEnd: function() {
+    _notifyPomodoroEnd() {
         if (this.notification &&
             this.notification instanceof Notifications.PomodoroEndNotification)
         {
-            /* do not renotify */
+            if (this.dialog && this.timer.isBreak()) {
+                this.dialog.open(true);
+            }
+            else if (this.notification.resident || this.notification.acknowledged) {
+                this.notification.show();
+            }
         }
         else {
             this.notification = new Notifications.PomodoroEndNotification(this.timer);
-            this.notification.connect('activated', Lang.bind(this,
-                function(notification) {
-                    if (this.dialog) {
-                        this.dialog.open(true);
-                        this.dialog.pushModal();
+            this.notification.connect('activated',
+                (notification) => {
+                    if (this.timer.isBreak()) {
+                        if (this.dialog) {
+                            this.dialog.open(true);
+                            this.dialog.pushModal();
+                        }
                     }
                     else {
-                        this.application.activate();
+                        this.timer.skip();
                     }
+                });
+            this.notification.connect('destroy', this._onNotificationDestroy.bind(this));
 
-                    if (this.reminder) {
-                        this.reminder.dismiss();
-                    }
-                }));
-            this.notification.connect('destroy', Lang.bind(this, this._onNotificationDestroy));
-
-            if (this.dialog) {
+            if (this.dialog && this.timer.isBreak()) {
                 this.dialog.open(true);
             }
             else {
                 this.notification.show();
-
-                if (this.reminder && !this.reminder.acknowledged) {
-                    this.reminder.schedule();
-                }
             }
 
             this._destroyPreviousNotifications();
         }
     },
 
-    _updateNotifications: function() {
-        if (this.timer.isPaused()) {
-            this._destroyNotifications();
+    _updateNotification() {
+        let timerState = this.timer.getState();
+        let isPaused   = this.timer.isPaused();
+
+        if (timerState != Timer.State.NULL && (!isPaused || this.timer.getElapsed() == 0.0)) {
+            if (this.mode == ExtensionMode.RESTRICTED) {
+                this._destroyNotifications();
+
+                if (!(this.notification &&
+                      this.notification instanceof Notifications.ScreenShieldNotification))
+                {
+                    this.notification = new Notifications.ScreenShieldNotification(this.timer);
+                    this.notification.connect('destroy', this._onNotificationDestroy.bind(this));
+                    this.notification.show();
+
+                    this._destroyPreviousNotifications();
+                }
+            }
+            else if (this.timer.getRemaining() > NOTIFICATIONS_TIME_OFFSET) {
+                if (timerState == Timer.State.POMODORO) {
+                    this._notifyPomodoroStart();
+                }
+                else {
+                    this._notifyPomodoroEnd();
+                }
+            }
+            else {
+                if (timerState != Timer.State.POMODORO) {
+                    this._notifyPomodoroStart();
+                }
+                else {
+                    this._notifyPomodoroEnd();
+                }
+            }
         }
         else {
-            switch (this.timer.getState()) {
-                case Timer.State.POMODORO:
-                    this._notifyPomodoroStart();
-                    break;
-
-                case Timer.State.SHORT_BREAK:
-                case Timer.State.LONG_BREAK:
-                    this._notifyPomodoroEnd();
-                    break;
-
-                default:
-                    this._destroyNotifications();
-                    break;
-            }
+            this._destroyNotifications();
         }
     },
 
-    _updateScreenNotifications: function() {
+    _updateScreenNotification() {
         if (this.dialog) {
             if (this.timer.isBreak() && !this.timer.isPaused()) {
                 this.dialog.open(false);
@@ -299,7 +347,7 @@ const PomodoroExtension = new Lang.Class({
         }
     },
 
-    _updatePresence: function() {
+    _updatePresence() {
         if (this.presence) {
             if (this._timerState == Timer.State.NULL) {
                 this.presence.setDefault();
@@ -310,33 +358,36 @@ const PomodoroExtension = new Lang.Class({
         }
     },
 
-    _update: function() {
+    _update() {
         let timerState = this.timer.getState();
+        let timerStateDuration = this.timer.getStateDuration();
         let isPaused = this.timer.isPaused();
-        let isRunning = timerState != Timer.State.NULL && !isPaused;
 
         if (this._isPaused != isPaused || this._timerState != timerState) {
             this._isPaused = isPaused;
             this._timerState = timerState;
-
-            if (this.reminder && !this.timer.isBreak()) {
-                this.reminder.unschedule();
-            }
+            this._timerStateDuration = timerStateDuration;
 
             this._updatePresence();
-            this._updateNotifications();
-            this._updateScreenNotifications();
+            this._updateNotification();
+            this._updateScreenNotification();
+        }
+        else if (this._timerStateDuration == timerStateDuration) {
+            this._updateScreenNotification();
+        }
+        else {
+            this._timerStateDuration = timerStateDuration;
         }
     },
 
-    _enableIndicator: function() {
+    _enableIndicator() {
         if (!this.indicator) {
             this.indicator = new Indicator.Indicator(this.timer,
                                                      this.pluginSettings.get_string('indicator-type'));
-            this.indicator.connect('destroy', Lang.bind(this,
-                function() {
+            this.indicator.connect('destroy',
+                () => {
                     this.indicator = null;
-                }));
+                });
 
             try {
                 Main.panel.addToStatusArea(Config.PACKAGE_NAME, this.indicator);
@@ -350,31 +401,31 @@ const PomodoroExtension = new Lang.Class({
         }
     },
 
-    _disableIndicator: function() {
+    _disableIndicator() {
         if (this.indicator) {
             this.indicator.actor.hide();
         }
     },
 
-    _enableKeybinding: function() {
+    _enableKeybinding() {
         if (!this.keybinding) {
             this.keybinding = true;
             Main.wm.addKeybinding('toggle-timer-key',
                                   this.settings,
                                   Meta.KeyBindingFlags.NONE,
                                   Shell.ActionMode.ALL,
-                                  Lang.bind(this, this._onKeybindingPressed));
+                                  this._onKeybindingPressed.bind(this));
         }
     },
 
-    _disableKeybinding: function() {
+    _disableKeybinding() {
         if (this.keybinding) {
             this.keybinding = false;
             Main.wm.removeKeybinding('toggle-timer-key');
         }
     },
 
-    _enablePresence: function() {
+    _enablePresence() {
         if (!this.presence) {
             this.presence = new Presence.Presence();
         }
@@ -382,51 +433,15 @@ const PomodoroExtension = new Lang.Class({
         this._updatePresence();
     },
 
-    _disablePresence: function() {
+    _disablePresence() {
         this._destroyPresence();
     },
 
-    _enableReminders: function() {
-        if (!this.reminder) {
-            this.reminder = new Notifications.ReminderManager(this.timer);
-            this.reminder.connect('notify', Lang.bind(this,
-                function() {
-                    let notification = new Notifications.RemindPomodoroEndNotification();
-                    notification.connect('activated', Lang.bind(this,
-                        function(notification) {
-                            this.reminder.dismiss();
-
-                            if (this.dialog) {
-                                this.dialog.open(true);
-                                this.dialog.pushModal();
-                            }
-                            else {
-                                this.application.activate();
-                            }
-
-                            notification.destroy();
-                        }));
-                }));
-            this.reminder.connect('destroy', Lang.bind(this,
-                function() {
-                    this.reminder = null;
-                }));
-        }
-
-        if (this.timer.isBreak() && !this.timer.isPaused() && this.reminder && !this.reminder.acknowledged) {
-            this.reminder.schedule();
-        }
-    },
-
-    _disableReminders: function() {
-        this._destroyReminders();
-    },
-
-    _enableScreenNotifications: function() {
+    _enableScreenNotification() {
         if (!this.dialog) {
             this.dialog = new Dialogs.PomodoroEndDialog(this.timer);
-            this.dialog.connect('opening', Lang.bind(this,
-                function() {
+            this.dialog.connect('opening', 
+                () => {
                     try {
                         if (Main.messageTray._notification) {
                             Main.messageTray._hideNotification(true);
@@ -435,13 +450,9 @@ const PomodoroExtension = new Lang.Class({
                     catch (error) {
                         Utils.logWarning(error.message);
                     }
-
-                    if (this.reminder) {
-                        this.reminder.unschedule();
-                    }
-                }));
-            this.dialog.connect('closing', Lang.bind(this,
-                function() {
+                });
+            this.dialog.connect('closing',
+                () => {
                     if (this.timer.isBreak() && !this.timer.isPaused()) {
                         if (this.notification instanceof Notifications.PomodoroEndNotification) {
                             this.notification.show();
@@ -451,73 +462,62 @@ const PomodoroExtension = new Lang.Class({
                             this.dialog.openWhenIdle();
                         }
                     }
-
-                    if (this.reminder && !this.reminder.acknowledged) {
-                        this.reminder.schedule();
-                    }
-                }));
-            this.dialog.connect('destroy', Lang.bind(this,
-                function() {
+                });
+            this.dialog.connect('destroy',
+                () => {
                     this.dialog = null;
-                }));
+                });
         }
 
-        this._updateScreenNotifications();
+        this._updateScreenNotification();
     },
 
-    _disableScreenNotifications: function() {
-        this._destroyScreenNotifications();
+    _disableScreenNotification() {
+        this._destroyScreenNotification();
     },
 
-    _destroyPresence: function() {
+    _destroyPresence() {
         if (this.presence) {
             this.presence.destroy();
             this.presence = null;
         }
     },
 
-    _destroyIndicator: function() {
+    _destroyIndicator() {
         if (this.indicator) {
             this.indicator.destroy();
             this.indicator = null;
         }
     },
 
-    _destroyReminders: function() {
-        if (this.reminder) {
-            this.reminder.destroy();
-            this.reminder = null;
-        }
-    },
-
-    _destroyNotifications: function() {
+    _destroyNotifications() {
         if (this.notificationSource) {
             this.notificationSource.destroyNotifications();
         }
     },
 
-    _destroyPreviousNotifications: function() {
+    _destroyPreviousNotifications() {
         if (this.notificationSource) {
             let notifications = this.notificationSource.notifications.filter(
-                Lang.bind(this, function(notification) {
+                (notification) => {
                     return notification !== this.notification;
-                }));
+                });
 
             notifications.forEach(
-                function(notification) {
+                (notification) => {
                     notification.destroy();
                 });
         }
     },
 
-    _destroyScreenNotifications: function() {
+    _destroyScreenNotification() {
         if (this.dialog) {
             this.dialog.destroy();
             this.dialog = null;
         }
     },
 
-    destroy: function() {
+    destroy() {
         if (this._destroying) {
             return;
         }
@@ -527,16 +527,15 @@ const PomodoroExtension = new Lang.Class({
 
         this._destroyPresence();
         this._destroyIndicator();
-        this._destroyScreenNotifications();
+        this._destroyScreenNotification();
         this._destroyNotifications();
-        this._destroyReminders();
 
         if (this.notificationSource) {
             this.notificationSource.destroy();
         }
 
         this.timer.destroy();
-
+        this.service.destroy();
         this.settings.run_dispose();
 
         this.emit('destroy');
@@ -552,38 +551,54 @@ function init(metadata) {
 
 
 function enable() {
+    let extension = Extension.extension;
     let sessionModeUpdatedId;
 
     if (!extension) {
         extension = new PomodoroExtension(Main.sessionMode.isLocked
                                           ? ExtensionMode.RESTRICTED : ExtensionMode.DEFAULT);
         extension.connect('destroy',
-            function() {
-                extension = null;
+            () => {
+                if (Extension.extension === extension) {
+                    Extension.extension = null;
+                }
 
                 if (sessionModeUpdatedId != 0) {
                     Main.sessionMode.disconnect(sessionModeUpdatedId);
                     sessionModeUpdatedId = 0;
                 }
             });
+        extension.connect('service-name-lost',
+            () => {
+                let metadata = ExtensionUtils.extensions[Config.EXTENSION_UUID];
 
-        sessionModeUpdatedId = Main.sessionMode.connect('updated',
-            function() {
-                if (Main.sessionMode.isLocked) {
-                    ExtensionSystem.enableExtension(Config.EXTENSION_UUID);
+                if (metadata && metadata.extension === extension) {
+                    ExtensionSystem.disableExtension(Config.EXTENSION_UUID);
                 }
                 else {
-                    extension.setMode(ExtensionMode.DEFAULT);
+                    extension.destroy();
                 }
             });
+
+        sessionModeUpdatedId = Main.sessionMode.connect('updated',
+            () => {
+                extension.setMode(Main.sessionMode.isLocked
+                                  ? ExtensionMode.RESTRICTED : ExtensionMode.DEFAULT);
+            });
+
+        Extension.extension = extension;
     }
 }
 
 
 function disable() {
+    let extension = Extension.extension;
+
     if (extension) {
-        if (Main.sessionMode.isLocked) {
+        if (Main.sessionMode.isLocked && !Main.sessionMode.isGreeter) {
             extension.setMode(ExtensionMode.RESTRICTED);
+
+            // Note that ExtensionSystem.disableExtension() will unload our styleshhet
         }
         else {
             extension.destroy();
