@@ -55,49 +55,45 @@ const FADE_OUT_OPACITY = 0.38;
 
 const STEPS = 120;
 
-const IndicatorType = {
+
+var IndicatorType = {
     TEXT: 'text',
     SHORT_TEXT: 'short-text',
     ICON: 'icon'
 };
 
 
-const IndicatorMenu = new Lang.Class({
+var IndicatorMenu = new Lang.Class({
     Name: 'PomodoroIndicatorMenu',
     Extends: PopupMenu.PopupMenu,
 
-    _init: function(indicator) {
+    _init(indicator) {
         this.parent(indicator.actor, St.Align.START, St.Side.TOP);
 
+        this._isPaused = null;
+        this._timerState = null;
         this._timerUpdateId = 0;
-        this._isRunning = false;
-        this._isPaused = false;
-        this._timerState = undefined;
 
         this.actor.add_style_class_name('extension-pomodoro-indicator-menu');
 
-        this._actorMappedId = this.actor.connect('notify::mapped', Lang.bind(this, this._onActorMapped));
+        this._actorMappedId = this.actor.connect('notify::mapped', this._onActorMapped.bind(this));
 
         this.indicator = indicator;
 
-        this._createTimerMenuItem();
-
-        this.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
-        this.addAction(_("Preferences"), Lang.bind(this, this._activatePreferences));
-        this.addAction(_("Quit"), Lang.bind(this, this._activateQuit));
+        this._populate();
     },
 
-    _createActionButton: function(iconName, accessibleName) {
+    _createActionButton(iconName, accessibleName) {
         let button = new St.Button({ reactive: true,
                                      can_focus: true,
                                      track_hover: true,
                                      accessible_name: accessibleName,
-                                     style_class: 'system-menu-action extension-pomodoro-menu-action' });
+                                     style_class: 'system-menu-action extension-pomodoro-indicator-menu-action' });
         button.child = new St.Icon({ icon_name: iconName });
         return button;
     },
 
-    _onTimerClicked: function() {
+    _onTimerClicked() {
         this.close();
 
         if (Extension.extension && Extension.extension.dialog) {
@@ -106,19 +102,19 @@ const IndicatorMenu = new Lang.Class({
         }
     },
 
-    _onStartClicked: function() {
+    _onStartClicked() {
         this.indicator.timer.start();
 
         this.close();
     },
 
-    _onStopClicked: function() {
+    _onStopClicked() {
         this.indicator.timer.stop();
 
         this.close();
     },
 
-    _onPauseClicked: function() {
+    _onPauseClicked() {
         if (!this.indicator.timer.isPaused ()) {
             this.indicator.timer.pause();
         }
@@ -129,63 +125,73 @@ const IndicatorMenu = new Lang.Class({
         }
     },
 
-    _createTimerMenuItem: function() {
-        let item;
-        let hbox;
+    _populate() {
+        let toggleItem = new PopupMenu.PopupMenuItem(_("Pomodoro Timer"),
+                                           { style_class: 'extension-pomodoro-indicator-menu-toggle',
+                                             reactive: false,
+                                             can_focus: false });
+        toggleItem.label.y_align = Clutter.ActorAlign.CENTER;
+        this.addMenuItem(toggleItem);
 
-        item = new PopupMenu.PopupMenuItem(_("Pomodoro Timer"),
-                                           { style_class: 'extension-pomodoro-menu-timer',
-                                            reactive: false,
-                                            can_focus: false });
-        item.label.y_align = Clutter.ActorAlign.CENTER;
+        let startAction = this._createActionButton('media-playback-start-symbolic', _("Start Timer"));
+        startAction.add_style_class_name('extension-pomodoro-indicator-menu-action-border');
+        startAction.connect('clicked', this._onStartClicked.bind(this));
+        toggleItem.actor.add(startAction);
 
-        this._timerMenuItem = item;
-        this._timerLabel = new St.Label({ style_class: 'extension-pomodoro-menu-timer-label',
-                                          y_align: Clutter.ActorAlign.CENTER });
-        this._timerLabelButton = new St.Button({ reactive: false,
-                                                 can_focus: false,
-                                                 track_hover: false,
-                                                 style_class: 'extension-pomodoro-menu-timer-label-button' });
-        this._timerLabelButton.child = this._timerLabel;
-        this._timerLabelButton.connect('clicked', Lang.bind(this, this._onTimerClicked));
+        let timerItem = new PopupMenu.PopupMenuItem("",
+                                           { style_class: 'extension-pomodoro-indicator-menu-timer',
+                                             reactive: false,
+                                             can_focus: false });
+        timerItem.label.visible = false;
+        this.addMenuItem(timerItem);
 
-        hbox = new St.BoxLayout();
+        let timerLabel = new St.Label({ style_class: 'extension-pomodoro-indicator-menu-timer-label',
+                                        y_align: Clutter.ActorAlign.CENTER });
+        let timerLabelButton = new St.Button({ reactive: false,
+                                               can_focus: false,
+                                               track_hover: false,
+                                               style_class: 'extension-pomodoro-indicator-menu-timer-label-button' });
+        timerLabelButton.child = timerLabel;
+        timerLabelButton.connect('clicked', this._onTimerClicked.bind(this));
+        timerItem.actor.add(timerLabelButton, { expand: true });
 
-        this._startAction = this._createActionButton('media-playback-start-symbolic', _("Start Timer"));
-        this._startAction.add_style_class_name('extension-pomodoro-menu-action-border');
-        this._startAction.connect('clicked', Lang.bind(this, this._onStartClicked));
-        hbox.add_actor(this._startAction);
+        let hbox = new St.BoxLayout();
+        timerItem.actor.add(hbox);
 
-        this._pauseAction = this._createActionButton('media-playback-pause-symbolic', _("Pause Timer"));
-        this._pauseAction.connect('clicked', Lang.bind(this, this._onPauseClicked));
-        hbox.add_actor(this._pauseAction);
+        let pauseAction = this._createActionButton('media-playback-pause-symbolic', _("Pause Timer"));
+        pauseAction.connect('clicked', this._onPauseClicked.bind(this));
+        hbox.add_actor(pauseAction);
 
-        this._stopAction = this._createActionButton('media-playback-stop-symbolic', _("Stop Timer"));
-        this._stopAction.connect('clicked', Lang.bind(this, this._onStopClicked));
-        hbox.add_actor(this._stopAction);
+        let stopAction = this._createActionButton('media-playback-stop-symbolic', _("Stop Timer"));
+        stopAction.connect('clicked', this._onStopClicked.bind(this));
+        hbox.add_actor(stopAction);
 
-        item.actor.add(this._timerLabelButton, { expand: true });
-        item.actor.add(hbox);
+        this._toggleMenuItem = toggleItem;
+        this._timerMenuItem = timerItem;
+        this._timerLabelButton = timerLabelButton;
 
-        this.addMenuItem(item);
-
-        this.timerLabel = this._timerLabel;
-        this.pauseAction = this._pauseAction;
+        this.timerLabel = timerLabel;
+        this.pauseAction = pauseAction;
 
         this.addStateMenuItem('pomodoro', _("Pomodoro"));
         this.addStateMenuItem('short-break', _("Short Break"));
         this.addStateMenuItem('long-break', _("Long Break"));
+
+        this.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
+
+        this.addAction(_("Preferences"), this._activatePreferences.bind(this));
+        this.addAction(_("Stats"), this._activateStats.bind(this));
+        this.addAction(_("Quit"), this._activateQuit.bind(this));
     },
 
-    addStateMenuItem: function(name, label) {
+    addStateMenuItem(name, label) {
         if (!this._stateItems) {
             this._stateItems = {};
         }
 
-        let menuItem = this.addAction(label, Lang.bind(this,
-            function(menuItem, event) {
+        let menuItem = this.addAction(label, (menuItem, event) => {
                 this._activateState(name);
-            }));
+            });
 
         menuItem.actor.add_style_class_name('state-item');
 
@@ -194,9 +200,9 @@ const IndicatorMenu = new Lang.Class({
         return menuItem;
     },
 
-    _onActorMapped: function(actor) {
+    _onActorMapped(actor) {
         if (actor.mapped && this._timerUpdateId == 0) {
-            this._timerUpdateId = this.indicator.timer.connect('update', Lang.bind(this, this._onTimerUpdate));
+            this._timerUpdateId = this.indicator.timer.connect('update', this._onTimerUpdate.bind(this));
             this._onTimerUpdate();
         }
 
@@ -206,40 +212,29 @@ const IndicatorMenu = new Lang.Class({
         }
     },
 
-    _onTimerUpdate: function() {
+    _onTimerUpdate() {
         let timer = this.indicator.timer;
         let timerState = timer.getState();
         let remaining = timer.getRemaining();
-        let isRunning = timerState != Timer.State.NULL;
         let isPaused = timer.isPaused();
+        let isRunning = timerState != Timer.State.NULL;
 
-        if (this._isRunning !== isRunning ||
-            this._isPaused !== isPaused ||
-            this._timerState !== timerState)
+        if (this._isPaused != isPaused ||
+            this._timerState != timerState)
         {
-            this._isRunning = isRunning;
             this._isPaused = isPaused;
             this._timerState = timerState;
 
-            this._timerMenuItem.label.visible = !isRunning;
-            this._timerLabel.visible = isRunning;
-            this._timerLabelButton.reactive = isRunning && !isPaused && timerState != Timer.State.POMODORO;
-            this._startAction.visible = !isRunning;
-            this._stopAction.visible = isRunning;
-            this._pauseAction.visible = isRunning;
-            this._pauseAction.child.icon_name = isPaused
-                                                ? 'media-playback-start-symbolic'
-                                                : 'media-playback-pause-symbolic';
-            this._pauseAction.accessible_name = isPaused
-                                                ? _("Resume Timer")
-                                                : _("Pause Timer");
+            this._toggleMenuItem.actor.visible = !isRunning;
+            this._timerMenuItem.actor.visible = isRunning;
 
-            if (isRunning) {
-                this._timerMenuItem.actor.add_style_class_name('extension-pomodoro-menu-timer-running');
-            }
-            else {
-                this._timerMenuItem.actor.remove_style_class_name('extension-pomodoro-menu-timer-running');
-            }
+            this._timerLabelButton.reactive = isRunning && !isPaused && timerState != Timer.State.POMODORO;
+            this.pauseAction.child.icon_name = isPaused
+                                               ? 'media-playback-start-symbolic'
+                                               : 'media-playback-pause-symbolic';
+            this.pauseAction.accessible_name = isPaused
+                                               ? _("Resume Timer")
+                                               : _("Pause Timer");
 
             for (let key in this._stateItems) {
                 let stateItem = this._stateItems[key];
@@ -260,7 +255,7 @@ const IndicatorMenu = new Lang.Class({
         this.timerLabel.set_text(this._formatTime(remaining));
     },
 
-    _formatTime: function(remaining) {
+    _formatTime(remaining) {
         if (remaining < 0.0) {
             remaining = 0.0;
         }
@@ -271,21 +266,27 @@ const IndicatorMenu = new Lang.Class({
         return '%02d:%02d'.format(minutes, seconds);
     },
 
-    _activatePreferences: function() {
+    _activateStats() {
+        let timestamp = global.get_current_time();
+
+        this.indicator.timer.showMainWindow('stats', timestamp);
+    },
+
+    _activatePreferences() {
         let timestamp = global.get_current_time();
 
         this.indicator.timer.showPreferences(timestamp);
     },
 
-    _activateQuit: function() {
+    _activateQuit() {
         this.indicator.timer.quit();
     },
 
-    _activateState: function(stateName) {
+    _activateState(stateName) {
         this.indicator.timer.setState(stateName);
     },
 
-    destroy: function() {
+    destroy() {
         if (this._timerUpdateId) {
             this.indicator.timer.disconnect(this._timerUpdateId);
             this._timerUpdateId = 0;
@@ -303,10 +304,10 @@ const IndicatorMenu = new Lang.Class({
 });
 
 
-const TextIndicator = new Lang.Class({
+var TextIndicator = new Lang.Class({
     Name: 'PomodoroTextIndicator',
 
-    _init : function(timer) {
+    _init(timer) {
         this._initialized     = false;
         this._state           = Timer.State.NULL;
         this._minHPadding     = 0;
@@ -325,22 +326,22 @@ const TextIndicator = new Lang.Class({
                                     y_align: Clutter.ActorAlign.CENTER });
         this.label.clutter_text.line_wrap = false;
         this.label.clutter_text.ellipsize = false;
-        this.label.connect('destroy', Lang.bind(this,
-            function() {
+        this.label.connect('destroy',
+            () => {
                 if (this._onTimerUpdateId) {
                     this.timer.disconnect(this._onTimerUpdateId);
                     this._onTimerUpdateId = 0;
                 }
-            }));
+            });
         this.actor.add_child(this.label);
 
-        this.actor.connect('get-preferred-width', Lang.bind(this, this._getPreferredWidth));
-        this.actor.connect('get-preferred-height', Lang.bind(this, this._getPreferredHeight));
-        this.actor.connect('allocate', Lang.bind(this, this._allocate));
-        this.actor.connect('style-changed', Lang.bind(this, this._onStyleChanged));
-        this.actor.connect('destroy', Lang.bind(this, this._onActorDestroy));
+        this.actor.connect('get-preferred-width', this._getPreferredWidth.bind(this));
+        this.actor.connect('get-preferred-height', this._getPreferredHeight.bind(this));
+        this.actor.connect('allocate', this._allocate.bind(this));
+        this.actor.connect('style-changed', this._onStyleChanged.bind(this));
+        this.actor.connect('destroy', this._onActorDestroy.bind(this));
 
-        this._onTimerUpdateId = this.timer.connect('update', Lang.bind(this, this._onTimerUpdate));
+        this._onTimerUpdateId = this.timer.connect('update', this._onTimerUpdate.bind(this));
 
         this._onTimerUpdate();
 
@@ -355,7 +356,7 @@ const TextIndicator = new Lang.Class({
         }
     },
 
-    _onStyleChanged: function(actor) {
+    _onStyleChanged(actor) {
         let themeNode = actor.get_theme_node();
         let font      = themeNode.get_font();
         let context   = actor.get_pango_context();
@@ -367,11 +368,11 @@ const TextIndicator = new Lang.Class({
         this._charWidth   = metrics.get_approximate_char_width() / Pango.SCALE;
     },
 
-    _getWidth: function() {
+    _getWidth() {
         return Math.ceil(4 * this._digitWidth + 0.5 * this._charWidth);
     },
 
-    _getPreferredWidth: function(actor, forHeight, alloc) {
+    _getPreferredWidth(actor, forHeight, alloc) {
         let child        = actor.get_first_child();
         let minWidth     = this._getWidth();
         let naturalWidth = minWidth;
@@ -395,7 +396,7 @@ const TextIndicator = new Lang.Class({
         }
     },
 
-    _getPreferredHeight: function(actor, forWidth, alloc) {
+    _getPreferredHeight(actor, forWidth, alloc) {
         let child = actor.get_first_child();
 
         if (child) {
@@ -406,7 +407,7 @@ const TextIndicator = new Lang.Class({
         }
     },
 
-    _getText: function(state, remaining) {
+    _getText(state, remaining) {
         if (remaining < 0.0) {
             remaining = 0.0;
         }
@@ -417,7 +418,7 @@ const TextIndicator = new Lang.Class({
         return '%02d:%02d'.format(minutes, seconds);
     },
 
-    _onTimerUpdate: function() {
+    _onTimerUpdate() {
         let state = this.timer.getState();
         let remaining = this.timer.getRemaining();
 
@@ -441,7 +442,7 @@ const TextIndicator = new Lang.Class({
         this.label.set_text(this._getText(state, remaining));
     },
 
-    _allocate: function(actor, box, flags) {
+    _allocate(actor, box, flags) {
         let child = actor.get_first_child();
         if (!child)
             return;
@@ -467,7 +468,7 @@ const TextIndicator = new Lang.Class({
         child.allocate(childBox, flags);
     },
 
-    _onActorDestroy: function() {
+    _onActorDestroy() {
         if (this._onTimerUpdateId) {
             this.timer.disconnect(this._onTimerUpdateId);
             this._onTimerUpdateId = 0;
@@ -478,29 +479,29 @@ const TextIndicator = new Lang.Class({
         this.emit('destroy');
     },
 
-    destroy: function() {
+    destroy() {
         this.actor.destroy();
     }
 });
 Signals.addSignalMethods(TextIndicator.prototype);
 
 
-const ShortTextIndicator = new Lang.Class({
+var ShortTextIndicator = new Lang.Class({
     Name: 'PomodoroShortTextIndicator',
     Extends: TextIndicator,
 
-    _init: function(timer) {
+    _init(timer) {
         this.parent(timer);
 
         this.label.set_x_align(Clutter.ActorAlign.END);
     },
 
-    _getWidth: function() {
+    _getWidth() {
         return Math.ceil(2 * this._digitWidth +
                          1 * this._charWidth);
     },
 
-    _getText: function(state, remaining) {
+    _getText(state, remaining) {
         if (remaining < 0.0) {
             remaining = 0.0;
         }
@@ -519,10 +520,10 @@ const ShortTextIndicator = new Lang.Class({
 });
 
 
-const IconIndicator = new Lang.Class({
+var IconIndicator = new Lang.Class({
     Name: 'PomodoroIconIndicator',
 
-    _init : function(timer) {
+    _init(timer) {
         this._state           = Timer.State.NULL;
         this._progress        = 0.0;
         this._minHPadding     = 0;
@@ -539,25 +540,25 @@ const IconIndicator = new Lang.Class({
         this.actor._delegate = this;
 
         this.icon = new St.DrawingArea({ style_class: 'system-status-icon' });
-        this.icon.connect('style-changed', Lang.bind(this, this._onIconStyleChanged));
-        this.icon.connect('repaint', Lang.bind(this, this._onIconRepaint));
-        this.icon.connect('destroy', Lang.bind(this, this._onIconDestroy));
+        this.icon.connect('style-changed', this._onIconStyleChanged.bind(this));
+        this.icon.connect('repaint', this._onIconRepaint.bind(this));
+        this.icon.connect('destroy', this._onIconDestroy.bind(this));
         this.actor.add_child(this.icon);
 
-        this.actor.connect('get-preferred-width', Lang.bind(this, this._getPreferredWidth));
-        this.actor.connect('get-preferred-height', Lang.bind(this, this._getPreferredHeight));
-        this.actor.connect('allocate', Lang.bind(this, this._allocate));
-        this.actor.connect('style-changed', Lang.bind(this, this._onStyleChanged));
-        this.actor.connect('destroy', Lang.bind(this, this._onActorDestroy));
+        this.actor.connect('get-preferred-width', this._getPreferredWidth.bind(this));
+        this.actor.connect('get-preferred-height', this._getPreferredHeight.bind(this));
+        this.actor.connect('allocate', this._allocate.bind(this));
+        this.actor.connect('style-changed', this._onStyleChanged.bind(this));
+        this.actor.connect('destroy', this._onActorDestroy.bind(this));
 
-        this._timerUpdateId = this.timer.connect('update', Lang.bind(this, this._onTimerUpdate));
+        this._timerUpdateId = this.timer.connect('update', this._onTimerUpdate.bind(this));
 
         this._onTimerUpdate();
 
         this._state = this.timer.getState();
     },
 
-    _onIconStyleChanged: function(actor) {
+    _onIconStyleChanged(actor) {
         let themeNode = actor.get_theme_node();
         let size = Math.ceil(themeNode.get_length('icon-size'));
 
@@ -567,7 +568,7 @@ const IconIndicator = new Lang.Class({
         this._iconSize = size;
     },
 
-    _onIconRepaint: function(area) {
+    _onIconRepaint(area) {
         let cr = area.get_context();
         let [width, height] = area.get_surface_size();
 
@@ -613,14 +614,14 @@ const IconIndicator = new Lang.Class({
         cr.$dispose();
     },
 
-    _onIconDestroy: function() {
+    _onIconDestroy() {
         if (this._timerUpdateId) {
             this.timer.disconnect(this._timerUpdateId);
             this._timerUpdateId = 0;
         }
     },
 
-    _onStyleChanged: function(actor) {
+    _onStyleChanged(actor) {
         let themeNode = actor.get_theme_node();
 
         this._minHPadding = themeNode.get_length('-minimum-hpadding');
@@ -638,7 +639,7 @@ const IconIndicator = new Lang.Class({
         });
     },
 
-    _getPreferredWidth: function(actor, forHeight, alloc) {
+    _getPreferredWidth(actor, forHeight, alloc) {
         let child = actor.get_first_child();
 
         if (child) {
@@ -652,7 +653,7 @@ const IconIndicator = new Lang.Class({
         alloc.natural_size += 2 * this._natHPadding;
     },
 
-    _getPreferredHeight: function(actor, forWidth, alloc) {
+    _getPreferredHeight(actor, forWidth, alloc) {
         let child = actor.get_first_child();
 
         if (child) {
@@ -666,7 +667,7 @@ const IconIndicator = new Lang.Class({
         alloc.natural_size += 2 * this._natVPadding;
     },
 
-    _onTimerUpdate: function() {
+    _onTimerUpdate() {
         let state = this.timer.getState();
         let progress = Math.floor(this.timer.getProgress() * STEPS) / STEPS;
 
@@ -677,7 +678,7 @@ const IconIndicator = new Lang.Class({
         }
     },
 
-    _allocate: function(actor, box, flags) {
+    _allocate(actor, box, flags) {
         let child = actor.get_first_child();
         if (!child) {
             return;
@@ -704,7 +705,7 @@ const IconIndicator = new Lang.Class({
         child.allocate(childBox, flags);
     },
 
-    _onActorDestroy: function() {
+    _onActorDestroy() {
         if (this._timerUpdateId) {
             this.timer.disconnect(this._timerUpdateId);
             this._timerUpdateId = 0;
@@ -718,25 +719,25 @@ const IconIndicator = new Lang.Class({
         this.emit('destroy');
     },
 
-    destroy: function() {
+    destroy() {
         this.actor.destroy();
     }
 });
 Signals.addSignalMethods(IconIndicator.prototype);
 
 
-const Indicator = new Lang.Class({
+var Indicator = new Lang.Class({
     Name: 'PomodoroIndicator',
     Extends: PanelMenu.Button,
 
-    _init: function(timer, type) {
+    _init(timer, type) {
         this.parent(St.Align.START, _("Pomodoro"), true);
 
         this.timer  = timer;
         this.widget = null;
 
         this.actor.add_style_class_name('extension-pomodoro-indicator');
-        this.actor.connect('destroy', Lang.bind(this, this._onActorDestroy));
+        this.actor.connect('destroy', this._onActorDestroy.bind(this));
 
         this._arrow = PopupMenu.arrowIcon(St.Side.BOTTOM);
         this._blinking = false;
@@ -744,9 +745,7 @@ const Indicator = new Lang.Class({
 
         this._hbox = new St.BoxLayout({ style_class: 'panel-status-menu-box' });
         this._hbox.pack_start = true;
-        this._hbox.add_child(this._arrow, { expand: false,
-                                            x_fill: false,
-                                            x_align: St.Align.END });
+        this._hbox.add_child(this._arrow);
         this.actor.add_child(this._hbox);
 
         this.setMenu(new IndicatorMenu(this));
@@ -754,11 +753,11 @@ const Indicator = new Lang.Class({
 
         this._onBlinked();
 
-        this._timerPausedId = this.timer.connect('paused', Lang.bind(this, this._onTimerPaused));
-        this._timerResumedId = this.timer.connect('resumed', Lang.bind(this, this._onTimerResumed));
+        this._timerPausedId = this.timer.connect('paused', this._onTimerPaused.bind(this));
+        this._timerResumedId = this.timer.connect('resumed', this._onTimerResumed.bind(this));
     },
 
-    setType: function(type) {
+    setType(type) {
         if (this.widget) {
             this.widget.destroy();
             this.widget = null;
@@ -783,12 +782,10 @@ const Indicator = new Lang.Class({
                                         'opacity',
                                         GObject.BindingFlags.SYNC_CREATE);
 
-        this._hbox.add_child(this.widget.actor, { expand: false,
-                                                  x_fill: false,
-                                                  x_align: St.Align.START });
+        this._hbox.add_child(this.widget.actor);
     },
 
-    _onBlinked: function() {
+    _onBlinked() {
         this._blinking = false;
 
         if (this.timer.isPaused()) {
@@ -796,7 +793,7 @@ const Indicator = new Lang.Class({
         }
     },
 
-    _blink: function() {
+    _blink() {
         if (!this._blinking) {
             this._blinking = true;
 
@@ -810,7 +807,7 @@ const Indicator = new Lang.Class({
                 transition: 'easeInOutQuad',
                 delay: FADE_OUT_TIME / 1000,
                 opacity: FADE_IN_OPACITY * 255,
-                onComplete: Lang.bind(this, this._onBlinked)
+                onComplete: this._onBlinked.bind(this)
             };
 
             if (Gtk.Settings.get_default().gtk_enable_animations) {
@@ -824,35 +821,35 @@ const Indicator = new Lang.Class({
             else if (this._blinkTimeoutSource == 0) {
                 Tweener.addTween(this._hbox, fadeOutParams);
 
-                this._blinkTimeoutSource = Mainloop.timeout_add (FADE_OUT_TIME, Lang.bind(this,
-                    function () {
+                this._blinkTimeoutSource = Mainloop.timeout_add(FADE_OUT_TIME,
+                    () => {
                         Tweener.addTween(this._hbox, fadeInParams);
 
-                        this._blinkTimeoutSource = Mainloop.timeout_add (FADE_IN_TIME, Lang.bind(this, function () {
+                        this._blinkTimeoutSource = Mainloop.timeout_add(FADE_IN_TIME, () => {
                             this._blinkTimeoutSource = 0;
 
                             this._onBlinked ();
 
                             return GLib.SOURCE_REMOVE;
-                        }));
+                        });
 
                         return GLib.SOURCE_REMOVE;
-                    }));
+                    });
             }
         }
     },
 
-    _onTimerPaused: function() {
+    _onTimerPaused() {
         this._blink();
     },
 
-    _onTimerResumed: function() {
+    _onTimerResumed() {
         if (this._blinking) {
             let fadeInParams = {
                 time: 200 / 1000,
                 transition: 'easeOutQuad',
                 opacity: FADE_IN_OPACITY * 255,
-                onComplete: Lang.bind(this, this._onBlinked)
+                onComplete: this._onBlinked.bind(this)
             };
 
             Tweener.removeTweens(this._hbox);
@@ -870,7 +867,7 @@ const Indicator = new Lang.Class({
         }
     },
 
-    _onActorDestroy: function() {
+    _onActorDestroy() {
         Tweener.removeTweens(this._hbox);
         Tweener.removeTweens(this.menu.timerLabel);
         Tweener.removeTweens(this.menu.pauseAction.child);
@@ -890,7 +887,7 @@ const Indicator = new Lang.Class({
         }
     },
 
-    destroy: function() {
+    destroy() {
         this.parent();
     }
 });
