@@ -30,25 +30,24 @@ const Utils = Extension.imports.utils;
 /**
  * Helps in managing presence for GNOME Shell according to the Pomodoro state.
  */
-const Presence = new Lang.Class({
+var Presence = new Lang.Class({
     Name: 'PomodoroPresence',
 
-    _init: function() {
+    _init() {
         this._busy = false;
 
         // Setup a patch for suppressing presence handlers.
         // When applied the main presence controller becomes gnome-pomodoro.
         this._patch = new Utils.Patch(MessageTray.MessageTray.prototype, {
-            _onStatusChanged:
-                function(status) {
-                    this._updateState();
-                }
+            _onStatusChanged(status) {
+                this._updateState();
+            }
         });
-        this._patch.connect('applied', Lang.bind(this, this._onPatchApplied));
-        this._patch.connect('reverted', Lang.bind(this, this._onPatchReverted));
+        this._patch.connect('applied', this._onPatchApplied.bind(this));
+        this._patch.connect('reverted', this._onPatchReverted.bind(this));
     },
 
-    setBusy: function(value) {
+    setBusy(value) {
         this._busy = value;
 
         if (!this._patch.applied) {
@@ -59,13 +58,13 @@ const Presence = new Lang.Class({
         }
     },
 
-    setDefault: function() {
+    setDefault() {
         if (this._patch.applied) {
             this._patch.revert();
         }
     },
 
-    _onPatchApplied: function() {
+    _onPatchApplied() {
         try {
             Main.messageTray._busy = this._busy;
             Main.messageTray._onStatusChanged();
@@ -75,7 +74,7 @@ const Presence = new Lang.Class({
         }
     },
 
-    _onPatchReverted: function() {
+    _onPatchReverted() {
         try {
             let status = Main.messageTray._presence.status;
             Main.messageTray._onStatusChanged(status);
@@ -85,7 +84,7 @@ const Presence = new Lang.Class({
         }
     },
 
-    destroy: function() {
+    destroy() {
         if (this._patch) {
             this._patch.destroy();
             this._patch = null;
