@@ -29,84 +29,84 @@ const DBus = Extension.imports.dbus;
 const Utils = Extension.imports.utils;
 
 
-const State = {
+var State = {
     NULL: 'null',
     POMODORO: 'pomodoro',
     SHORT_BREAK: 'short-break',
-    LONG_BREAK: 'long-break'
+    LONG_BREAK: 'long-break',
+
+    label(state) {
+        switch (state) {
+            case State.POMODORO:
+                return _("Pomodoro");
+
+            case State.SHORT_BREAK:
+                return _("Short Break");
+
+            case State.LONG_BREAK:
+                return _("Long Break");
+
+            default:
+                return null;
+        }
+    }
 };
 
 
-const Timer = new Lang.Class({
+var Timer = new Lang.Class({
     Name: 'PomodoroTimer',
 
-    _init: function() {
+    _init() {
         this._connected = false;
         this._state = null;
         this._isPaused = null;
+        this._stateDuration = 0;
         this._propertiesChangedId = 0;
-        this._settingsChangedId = 0;
-        this._shortBreakDuration = 0;
-        this._longBreakDuration = 0;
+        this._elapsed = 0.0;
 
-        let settings = Extension.extension.settings;
-        this._settingsChangedId  = settings.connect('changed', Lang.bind(this, this._onSettingsChanged));
-        this._shortBreakDuration = settings.get_double('short-break-duration');
-        this._longBreakDuration  = settings.get_double('long-break-duration');
-
-        this._proxy = DBus.Pomodoro(Lang.bind(this, function(proxy, error) {
-            if (error) {
-                Utils.logWarning(error.message);
-                this._notifyServiceNotInstalled();
-                return;
-            }
-        }));
+        this._proxy = DBus.Pomodoro(this._onInit.bind(this));
 
         this._propertiesChangedId = this._proxy.connect(
                                        'g-properties-changed',
-                                       Lang.bind(this, this._onPropertiesChanged));
+                                       this._onPropertiesChanged.bind(this));
 
         this._nameWatcherId = Gio.DBus.session.watch_name(
                                        'org.gnome.Pomodoro',
                                        Gio.BusNameWatcherFlags.AUTO_START,
-                                       Lang.bind(this, this._onNameAppeared),
-                                       Lang.bind(this, this._onNameVanished));
+                                       this._onNameAppeared.bind(this),
+                                       this._onNameVanished.bind(this));
     },
 
-    _onSettingsChanged: function(settings, key) {
-        switch (key) {
-            case 'short-break-duration':
-                this._shortBreakDuration = settings.get_double('short-break-duration');
-                break;
-
-            case 'long-break-duration':
-                this._longBreakDuration = settings.get_double('long-break-duration');
-                break;
-        }
-    },
-
-    _onNameAppeared: function() {
+    _onNameAppeared() {
         this._connected = true;
 
         this.emit('service-connected');
         this.emit('update');
     },
 
-    _onNameVanished: function() {
+    _onNameVanished() {
         this._connected = false;
 
         this.emit('update');
         this.emit('service-disconnected');
     },
 
-    _onPropertiesChanged: function(proxy, properties) {
+    _onPropertiesChanged(proxy, properties) {
         let state = proxy.State;
+        let stateDuration = proxy.StateDuration;
+        let elapsed = proxy.Elapsed;
         let isPaused = proxy.IsPaused;
 
-        if (this._state !== state) {
+        if (this._state != state || this._stateDuration != stateDuration || this._elapsed > elapsed) {
             this._state = state;
+            this._stateDuration = stateDuration;
+            this._elapsed = elapsed
+
             this.emit('state-changed');
         }
+        else {
+            this._elapsed = elapsed;
+        }
 
         if (this._isPaused !== isPaused) {
             this._isPaused = isPaused;
@@ -116,7 +116,14 @@ const Timer = new Lang.Class({
         this.emit('update');
     },
 
-    _onCallback: function(result, error) {
+    _onInit(proxy, error) {
+        if (error) {
+            Utils.logWarning(error.message);
+            this._notifyServiceNotInstalled();
+        }
+    },
+
+    _onCallback(result, error) {
         if (error) {
             Utils.logWarning(error.message);
 
@@ -126,74 +133,90 @@ const Timer = new Lang.Class({
         }
     },
 
-    getState: function() {
-        if (!this._connected || this._proxy.State == null) {
+    getState() {
+        if (!this._connected || this._proxy.State === null) {
             return State.NULL;
         }
 
         return this._proxy.State;
     },
 
-    setState: function(state, duration) {
+    setState(state, timestamp) {
         this._proxy.SetStateRemote(state,
-                                   duration || 0,
-                                   Lang.bind(this, this._onCallback));
+                                   timestamp || 0,
+                                   this._onCallback.bind(this));
     },
 
-    getStateDuration: function() {
+    getStateDuration() {
         return this._proxy.StateDuration;
     },
 
-    getElapsed: function() {
+    setStateDuration(duration) {
+        this._proxy.SetStateDurationRemote(this._proxy.State,
+                                           duration,
+                                           this._onCallback.bind(this));
+    },
+
+    get stateDuration() {
+        return this._proxy.StateDuration;
+    },
+
+    set stateDuration(value) {
+        this._proxy.SetStateDurationRemote(this._proxy.State,
+                                           value,
+                                           this._onCallback.bind(this));
+    },
+
+    getElapsed() {
         return this._proxy.Elapsed;
     },
 
-    getRemaining: function() {
+    getRemaining() {
         let state = this.getState();
 
-        if (state == State.NULL) {
+        if (state === State.NULL) {
             return 0.0;
         }
 
         return Math.ceil(this._proxy.StateDuration - this._proxy.Elapsed);
     },
 
-    getProgress: function() {
+    getProgress() {
         return (this._connected && this._proxy.StateDuration > 0)
                 ? this._proxy.Elapsed / this._proxy.StateDuration
                 : 0.0;
     },
 
-    isPaused: function() {
+    isPaused() {
         return this._connected && this._proxy.IsPaused;
     },
 
-    start: function() {
-        this._proxy.StartRemote(Lang.bind(this, this._onCallback));
+    start() {
+        this._proxy.StartRemote(this._onCallback.bind(this));
     },
 
-    stop: function() {
-        this._proxy.StopRemote(Lang.bind(this, this._onCallback));
+    stop() {
+        this._proxy.StopRemote(this._onCallback.bind(this));
     },
 
-    pause: function() {
-        this._proxy.PauseRemote(Lang.bind(this, this._onCallback));
+    pause() {
+        this._proxy.PauseRemote(this._onCallback.bind(this));
     },
 
-    resume: function() {
-        this._proxy.ResumeRemote(Lang.bind(this, this._onCallback));
+    resume() {
+        this._proxy.ResumeRemote(this._onCallback.bind(this));
     },
 
-    skip: function() {
-        this._proxy.SkipRemote(Lang.bind(this, this._onCallback));
+    skip() {
+        this._proxy.SkipRemote(this._onCallback.bind(this));
     },
 
-    reset: function() {
-        this._proxy.ResetRemote(Lang.bind(this, this._onCallback));
+    reset() {
+        this._proxy.ResetRemote(this._onCallback.bind(this));
     },
 
-    toggle: function() {
-        if (this.getState() == State.NULL) {
+    toggle() {
+        if (this.getState() === State.NULL) {
             this.start();
         }
         else {
@@ -201,48 +224,31 @@ const Timer = new Lang.Class({
         }
     },
 
-    isBreak: function() {
+    isBreak() {
         let state = this.getState();
 
-        return state == State.SHORT_BREAK || state == State.LONG_BREAK;
+        return state === State.SHORT_BREAK || state === State.LONG_BREAK;
     },
 
-    canSwitchBreak: function() {
-        return (this.getElapsed() < this._shortBreakDuration) &&
-               (this._shortBreakDuration < this._longBreakDuration);
+    showMainWindow(timestamp) {
+        this._proxy.ShowMainWindowRemote(timestamp, this._onCallback.bind(this));
     },
 
-    switchBreak: function() {
-        let state = this.getState();
-
-        if (state == State.SHORT_BREAK) {
-            this.setState(State.LONG_BREAK);
-        }
-
-        if (state == State.LONG_BREAK) {
-            this.setState(State.SHORT_BREAK);
-        }
+    showPreferences(timestamp) {
+        this._proxy.ShowPreferencesRemote(timestamp, this._onCallback.bind(this));
     },
 
-    showMainWindow: function(timestamp) {
-        this._proxy.ShowMainWindowRemote(timestamp, Lang.bind(this, this._onCallback));
-    },
-
-    showPreferences: function(timestamp) {
-        this._proxy.ShowPreferencesRemote(timestamp, Lang.bind(this, this._onCallback));
-    },
-
-    quit: function() {
-        this._proxy.QuitRemote(Lang.bind(this, function(result, error) {
+    quit() {
+        this._proxy.QuitRemote((result, error) => {
             Utils.disableExtension(Config.EXTENSION_UUID);
-        }));
+        });
     },
 
-    _notifyServiceNotInstalled: function() {
+    _notifyServiceNotInstalled() {
         Extension.extension.notifyIssue(_("Failed to run <i>%s</i> service").format(Config.PACKAGE_NAME));
     },
 
-    destroy: function() {
+    destroy() {
         if (this._propertiesChangedId != 0) {
             this._proxy.disconnect(this._propertiesChangedId);
             this._propertiesChangedId = 0;
@@ -252,11 +258,6 @@ const Timer = new Lang.Class({
             Gio.DBus.session.unwatch_name(this._nameWatcherId);
             this._nameWatcherId = 0;
         }
-
-        if (this._settingsChangedId) {
-            Extension.extension.settings.disconnect(this._settingsChangedId);
-            this._settingsChangedId = 0;
-        }
     }
 });
 Signals.addSignalMethods(Timer.prototype);
